import os
import pathlib
import tempfile
from typing import Dict, Generator, List, Tuple

import pytest

from vessl.util.vesslignore import list_files_using_vesslignore


@pytest.fixture()
def vesslignore_test_files(tmp_path: pathlib.Path):
    """
    Pytest fixture function that prepares and removes files for .vesslignore test.
    """

    ignore_contents: Dict[str, List[str]] = {}
    files_to_write: List[Tuple[str, bool]] = [] # (path, should_exist)

    def _add_ignore(pattern, write_to='.vesslignore'):
        if write_to not in ignore_contents:
            ignore_contents[write_to] = []
        ignore_contents[write_to].append(pattern)

    def _expect_present(filename):
        nonlocal files_to_write
        files_to_write.append((filename, True))

    def _expect_ignored(filename):
        nonlocal files_to_write
        files_to_write.append((filename, False))

    # Comment
    _add_ignore('# Comments should work')

    # Simple bans
    _add_ignore('*.o')
    _add_ignore('.*.sw?') # Vim swap files -- to test '?'
    _expect_present('SHOWN')
    _expect_ignored('HIDDEN.o')
    _expect_ignored('.HIDDEN.swp')
    _expect_ignored('.HIDDEN.swo')
    _expect_present('SHOWN.obj') # do not allow the partial match 'SHOWN.o'
    # Simple bans in subdirectory
    _expect_present('data1/SHOWN')
    _expect_ignored('data1/HIDDEN.o')
    _expect_ignored('data1/.HIDDEN.swp')
    _expect_ignored('data1/.HIDDEN.swo')
    # Simple bans should match directory names, too
    _add_ignore('HIDDEN_DIR?')
    _expect_ignored('data1/HIDDEN_DIR3/HIDDEN.txt')

    # Patterns with '**/' should match in all places
    _add_ignore('**/animal/cat')
    _expect_present('data2/animal/dog')
    _expect_ignored('data2/animal/cat')
    _expect_ignored('data2/dist/animal/cat/mango') # directory match
    _expect_present('data2/animal/catto') # basename should match exactly, not prefixed
    # Trailing slash should force directory match
    _add_ignore('**/fruit/apples/')
    _expect_present('data2/fruit/apples')
    _expect_ignored('data2/dist/fruit/apples/Macintosh')

    # Relative path from root
    _add_ignore('data3/instrument/viol*')
    _expect_present('data3/instrument/trumpet')
    _expect_ignored('data3/instrument/violin')
    _expect_ignored('data3/instrument/viola/3') # directory match
    # wildcard inside path
    _add_ignore('data3/f?sh/*/tail')
    _expect_present('data3/fish/tuna/head')
    _expect_ignored('data3/fish/tuna/tail')
    _expect_present('data3/fish/tuna/tailor') # basename should not be prefixed
    # Trailing slash should force directory match
    _add_ignore('data3/fish/*/head/')
    _expect_present('data3/fish/cod/heading')
    _expect_present('data3/fish/cod/head')
    _expect_ignored('data3/fish/octopus/head/eye')

    # .vesslignore in subdirectory
    def _add_ignore_sub(pattern):
        _add_ignore(pattern, 'subdir/.vesslignore')

    # Subdirectory pattern test
    _add_ignore_sub('cat')
    _add_ignore_sub('**/SHOWN')
    _add_ignore_sub('SUB_IGNORE/')
    _expect_ignored('subdir/cat')
    _expect_present('subdir/catto')
    _expect_ignored('subdir/data1/HIDDEN.o')
    _expect_present('subdir/data2/SUB_IGNORE/blob')
    _expect_present('subdir/data3/SUB_IGNORE')
    _expect_ignored('subdir/SUB_IGNORE/blob')
    _expect_present('SUB_IGNORE/SHOWN') # subdir pattern should not affect parent

    for filename, present in files_to_write:
        path = tmp_path / filename
        path.parent.mkdir(exist_ok=True, parents=True)
        path.touch()
        pass

    for ignore_filename, ignore_lines in ignore_contents.items():
        path = tmp_path / ignore_filename
        path.parent.mkdir(exist_ok=True, parents=True)

        ignore_content = '\n'.join(ignore_lines) + '\n'
        path.open('w').write(ignore_content)

    yield (tmp_path, files_to_write)

class TestVesslIgnore:
    def test_list_files_using_vesslignore(self, vesslignore_test_files: Tuple[pathlib.Path, List[Tuple[str, bool]]]):
        root_path, files_to_write = vesslignore_test_files

        files_found = list_files_using_vesslignore(str(root_path))

        for filename, should_exist in files_to_write:
            assert (filename in files_found) == should_exist



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseDashboardExperimentFilterResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'field': 'ResponseDashboardExperimentField',
        'id': 'int',
        'is_active': 'bool',
        'operator': 'str',
        'values': 'ResponseDashboardExperimentFilterValues'
    }

    attribute_map = {
        'field': 'field',
        'id': 'id',
        'is_active': 'is_active',
        'operator': 'operator',
        'values': 'values'
    }

    def __init__(self, field=None, id=None, is_active=None, operator=None, values=None, local_vars_configuration=None):  # noqa: E501
        """ResponseDashboardExperimentFilterResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._field = None
        self._id = None
        self._is_active = None
        self._operator = None
        self._values = None
        self.discriminator = None

        self.field = field
        self.id = id
        self.is_active = is_active
        self.operator = operator
        self.values = values

    @property
    def field(self):
        """Gets the field of this ResponseDashboardExperimentFilterResponse.  # noqa: E501


        :return: The field of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :rtype: ResponseDashboardExperimentField
        """
        return self._field

    @field.setter
    def field(self, field):
        """Sets the field of this ResponseDashboardExperimentFilterResponse.


        :param field: The field of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :type field: ResponseDashboardExperimentField
        """
        if self.local_vars_configuration.client_side_validation and field is None:  # noqa: E501
            raise ValueError("Invalid value for `field`, must not be `None`")  # noqa: E501

        self._field = field

    @property
    def id(self):
        """Gets the id of this ResponseDashboardExperimentFilterResponse.  # noqa: E501


        :return: The id of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseDashboardExperimentFilterResponse.


        :param id: The id of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def is_active(self):
        """Gets the is_active of this ResponseDashboardExperimentFilterResponse.  # noqa: E501


        :return: The is_active of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :rtype: bool
        """
        return self._is_active

    @is_active.setter
    def is_active(self, is_active):
        """Sets the is_active of this ResponseDashboardExperimentFilterResponse.


        :param is_active: The is_active of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :type is_active: bool
        """
        if self.local_vars_configuration.client_side_validation and is_active is None:  # noqa: E501
            raise ValueError("Invalid value for `is_active`, must not be `None`")  # noqa: E501

        self._is_active = is_active

    @property
    def operator(self):
        """Gets the operator of this ResponseDashboardExperimentFilterResponse.  # noqa: E501


        :return: The operator of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :rtype: str
        """
        return self._operator

    @operator.setter
    def operator(self, operator):
        """Sets the operator of this ResponseDashboardExperimentFilterResponse.


        :param operator: The operator of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :type operator: str
        """
        if self.local_vars_configuration.client_side_validation and operator is None:  # noqa: E501
            raise ValueError("Invalid value for `operator`, must not be `None`")  # noqa: E501

        self._operator = operator

    @property
    def values(self):
        """Gets the values of this ResponseDashboardExperimentFilterResponse.  # noqa: E501


        :return: The values of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :rtype: ResponseDashboardExperimentFilterValues
        """
        return self._values

    @values.setter
    def values(self, values):
        """Sets the values of this ResponseDashboardExperimentFilterResponse.


        :param values: The values of this ResponseDashboardExperimentFilterResponse.  # noqa: E501
        :type values: ResponseDashboardExperimentFilterValues
        """
        if self.local_vars_configuration.client_side_validation and values is None:  # noqa: E501
            raise ValueError("Invalid value for `values`, must not be `None`")  # noqa: E501

        self._values = values

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseDashboardExperimentFilterResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseDashboardExperimentFilterResponse):
            return True

        return self.to_dict() != other.to_dict()
